package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.BasicExamStudentQuery;
import com.qmth.distributed.print.business.bean.dto.BasicExamStudentDto;
import com.qmth.distributed.print.business.bean.dto.DeleteBasicExamStudentStatusDto;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.params.BasicExamStudentParam;
import com.qmth.distributed.print.business.bean.result.BasicExamStudentResult;
import com.qmth.distributed.print.business.entity.BasicExamRule;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.enums.RequiredFieldsEnum;
import com.qmth.distributed.print.business.mapper.BasicExamStudentMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableDisabledValue;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.BasicExamStudentStatusEnum;
import com.qmth.teachcloud.common.enums.EnumResult;
import com.qmth.teachcloud.common.enums.ExamModelEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkStudentService;
import com.qmth.teachcloud.mark.service.MarkTaskService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 考生字典表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
@Service
public class BasicExamStudentServiceImpl extends ServiceImpl<BasicExamStudentMapper, BasicExamStudent> implements BasicExamStudentService {

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private TeachCourseService teachCourseService;
    @Resource
    private CommonCacheService commonCacheService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private BasicExamService basicExamService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private ExamStudentService examStudentService;

    @Override
    public IPage<BasicExamStudentResult> page(Long semesterId, Long examId, Long courseId, String paperNumber,
                                              String teacher, String college, String major, String teachClassName, String className, String examStudentInfo, int pageNumber,
                                              int pageSize) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();

        teacher = SystemConstant.translateSpecificSign(teacher);
        college = SystemConstant.translateSpecificSign(college);
        major = SystemConstant.translateSpecificSign(major);
        examStudentInfo = SystemConstant.translateSpecificSign(examStudentInfo);
        teachClassName = SystemConstant.translateSpecificSign(teachClassName);
        className = SystemConstant.translateSpecificSign(className);

        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(requestUser.getSchoolId(), requestUser.getId(),
                ServletUtil.getRequest().getServletPath());
        IPage<BasicExamStudentResult> page = this.baseMapper.findBasicExamStudentPage(new Page<>(pageNumber, pageSize),
                requestUser.getSchoolId(), semesterId, examId, courseId, paperNumber, teacher, college, major, teachClassName, className, examStudentInfo, dpr);
        for (BasicExamStudentResult record : page.getRecords()) {
            Long examStartTime = record.getExamStartTime();
            Long examEndTime = record.getExamEndTime();
            if (SystemConstant.longNotNull(examStartTime) && SystemConstant.longNotNull(examEndTime)) {
                Map<String, Object> dateMap = ConvertUtil.analyzeDateAndTime(examStartTime, examEndTime);
                String date = String.valueOf(dateMap.get("date"));
                String time = String.valueOf(dateMap.get("time"));
                record.setExamDate(date);
                record.setExamTime(time);
            }
        }
        return page;
    }

    @Override
    public List<BasicExamStudentResult> list(SysUser requestUser, Long semesterId, Long examId, Long courseId,
                                             String teacher, String college, String major, String teachClazz, String examStudentInfo) {
        Long schoolId = requestUser.getSchoolId();

        teacher = SystemConstant.translateSpecificSign(teacher);
        college = SystemConstant.translateSpecificSign(college);
        major = SystemConstant.translateSpecificSign(major);
        examStudentInfo = SystemConstant.translateSpecificSign(examStudentInfo);
        teachClazz = SystemConstant.translateSpecificSign(teachClazz);

        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(),
                ServletUtil.getRequest().getServletPath());
        List<BasicExamStudentResult> list = this.baseMapper.findBasicExamStudentList(schoolId, semesterId, examId,
                courseId, teacher, college, major, teachClazz, examStudentInfo, dpr);
        for (BasicExamStudentResult basicExamStudentResult : list) {
            Long examStartTime = basicExamStudentResult.getExamStartTime();
            Long examEndTime = basicExamStudentResult.getExamEndTime();
            if (SystemConstant.longNotNull(examStartTime) && SystemConstant.longNotNull(examEndTime)) {
                Map<String, Object> dateMap = ConvertUtil.analyzeDateAndTime(examStartTime, examEndTime);
                String date = String.valueOf(dateMap.get("date"));
                String time = String.valueOf(dateMap.get("time"));
                basicExamStudentResult.setExamDate(date);
                basicExamStudentResult.setExamTime(time);
            }
        }
        return list;
    }

    @Transactional
    @Override
    public Long save(BasicExamStudentParam basicExamStudentParam, SysUser requestUser) {
        BasicExam basicExam = basicExamService.getById(basicExamStudentParam.getExamId());
        BasicExamStudent basicExamStudent = this.editEntityHelp(basicExamStudentParam, basicExam, requestUser);
        this.saveOrUpdate(basicExamStudent);
        // 更新扫描阅卷数据
        this.updateMarkData(basicExam.getId(), Arrays.asList(basicExamStudent), requestUser.getId());
        // 自动创建课程管理数据
        teachCourseService.batchCreateTeachCourse(basicExamStudent.getSchoolId(), basicExamStudent.getExamId(), Arrays.asList(basicExamStudent));
        return basicExamStudent.getId();
    }

    @Override
    public void exportLogic(HttpServletResponse response, Long semesterId, Long examId, Long courseId, String teacher, String college, String major, String teachClazz, String examStudentInfo)
            throws Exception {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        List<BasicExamStudentResult> list = this.list(requestUser, semesterId, examId, courseId, teacher, college,
                major, teachClazz, examStudentInfo);
        List<BasicExamStudentDto> exportDto = list.stream().flatMap(e -> {
            BasicExamStudentDto basicExamStudentDto = new BasicExamStudentDto();
            BeanUtils.copyProperties(e, basicExamStudentDto);
            return Stream.of(basicExamStudentDto);
        }).collect(Collectors.toList());

        // 生成excel文件
        ExcelUtil.excelExport("考生字典数据", BasicExamStudentDto.class, exportDto, response);
    }

    @Transactional
    @Override
    public void deleteBatch(List<Long> idList) {
        if (CollectionUtils.isNotEmpty(idList)) {
            List<BasicExamStudent> basicExamStudentList = this.listByIds(idList);
            for (BasicExamStudent basicExamStudent : basicExamStudentList) {
                List<MarkStudent> markStudentList = markStudentService.listByStudentId(basicExamStudent.getId());
                if (CollectionUtils.isNotEmpty(markStudentList)) {
                    StringJoiner stringJoiner = new StringJoiner(";");
                    for (MarkStudent markStudent : markStudentList) {
                        if (markTaskService.countByStudentId(markStudent.getId()) > 0) {
                            BasicCourse basicCourse = basicCourseService.getById(markStudent.getCourseId());
                            stringJoiner.add("课程[" + basicCourse.getName() + "(" + basicCourse.getCode() + ")]，学号[" + basicExamStudent.getStudentCode() + "]考生已开始评卷，不允许删除");
                        }
                    }
                    if (stringJoiner.toString().length() > 0) {
                        throw ExceptionResultEnum.ERROR.exception(stringJoiner.toString());
                    }
                }
                examStudentService.deleteByBasicStudentId(basicExamStudent.getId());
                markStudentService.deleteByBasicStudentId(basicExamStudent.getId());
                // 更新扫描数据
                long count = markStudentService.countByExamIdAndPaperNumber(basicExamStudent.getExamId(), basicExamStudent.getPaperNumber(), null);
                if (count == 0) {
                    markPaperService.deleteByExamIdAndPaperNumber(basicExamStudent.getExamId(), basicExamStudent.getPaperNumber());
                } else {
                    markPaperService.updateStudentCountByExamIdAndPaperNumberAndPaperType(basicExamStudent.getExamId(), basicExamStudent.getPaperNumber(), null);
                    markPaperService.updateUploadCount(basicExamStudent.getExamId(), basicExamStudent.getPaperNumber(),
                            markStudentService.countUploadedByExamIdAndPaperNumber(basicExamStudent.getExamId(), basicExamStudent.getPaperNumber()));
                }
            }
            this.removeByIds(idList);
        }
    }

    /**
     * 构造考生新增/编辑对象
     *
     * @param basicExamStudentParam 考生参数
     * @param requestUser           请求人
     * @return 考生对象
     */
    @Override
    public BasicExamStudent editEntityHelp(BasicExamStudentParam basicExamStudentParam, BasicExam basicExam, SysUser requestUser) {
        List<String> errorMsgList = new ArrayList<>();
        Long schoolId = requestUser.getSchoolId();
        Long requestUserId = requestUser.getId();

        String paperNumber = basicExamStudentParam.getPaperNumber();
        String teacherName = basicExamStudentParam.getTeacherName();
        String teacherCode = basicExamStudentParam.getTeacherCode();
        String paperType = basicExamStudentParam.getPaperType();

        Long teacherId = null;
        if (SystemConstant.strNotNull(teacherCode) && !SystemConstant.strNotNull(teacherName)) {
            errorMsgList.add("任课老师必填");
        }
        if (SystemConstant.strNotNull(teacherName) && !SystemConstant.strNotNull(teacherCode)) {
            errorMsgList.add("任课老师工号必填");
        }
        if (StringUtils.isNotBlank(teacherCode) && StringUtils.isNotBlank(teacherName)) {
            SysUser sysUser = sysUserService.getByLoginName(schoolId, teacherCode);
            if (sysUser == null) {
                errorMsgList.add("任课老师工号[" + teacherCode + "]用户不存在");
            } else {
                if (!sysUser.getEnable()) {
                    errorMsgList.add("任课老师工号[" + teacherCode + "]用户已禁用");
                } else if (!sysUser.getRealName().equals(teacherName)) {
                    errorMsgList.add("任课老师[" + teacherName + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                } else {
                    teacherId = sysUser.getId();
                }
            }
        }

        // 校验试卷编号存在时，任课教师，考试时间，考点考场均必填；
        BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
        boolean needPaperNumber = (basicSchool.getHasPaperNumber() != null && basicSchool.getHasPaperNumber()) || ExamModelEnum.MODEL4.equals(basicExam.getExamModel());
        if (needPaperNumber && StringUtils.isBlank(paperNumber)) {
            errorMsgList.add("试卷编号必填");
        }

        if (CollectionUtils.isNotEmpty(errorMsgList)) {
            throw ExceptionResultEnum.ERROR.exception(String.join(";", errorMsgList));
        }

        // 校验试卷编号和卷型
        if (!ExamModelEnum.MODEL4.equals(basicExam.getExamModel()) && StringUtils.isNoneBlank(basicExamStudentParam.getPaperNumber(), basicExamStudentParam.getPaperType())) {
            // 不传课程ID
            ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamIdAndCourseIdAndPaperNumber(basicExamStudentParam.getExamId(), null, basicExamStudentParam.getPaperNumber());
            if (examTaskDetail != null) {
                if (!examTaskDetail.getPaperType().contains(basicExamStudentParam.getPaperType())) {
                    throw ExceptionResultEnum.ERROR.exception("试卷编号[" + basicExamStudentParam.getPaperNumber() + "]的命题任务卷型为[" + examTaskDetail.getPaperType() + "]");
                }
            }
        }

        BasicExamStudent basicExamStudent = new BasicExamStudent();
        BeanUtils.copyProperties(basicExamStudentParam, basicExamStudent);
        basicExamStudent.setSchoolId(schoolId);
        basicExamStudent.setTeacherId(teacherId);

        // 同一课程+教学班下只能有一位任课老师
        List<BasicExamStudent> basicExamStudentList = this.listByExamIdAndCourseIdAndTeachClassName(basicExamStudent.getExamId(), basicExamStudent.getCourseId(), basicExamStudent.getTeachClassName());
//        if (CollectionUtils.isNotEmpty(basicExamStudentList)) {
//            List<Long> teacherIds = basicExamStudentList.stream().filter(m -> m.getTeacherId() != null).map(BasicExamStudent::getTeacherId).distinct().collect(Collectors.toList());
//            if (teacherIds.size() > 1 || (basicExamStudent.getTeacherId() != null && teacherIds.get(0) != null && basicExamStudent.getTeacherId().equals(teacherIds.get(0)))) {
//                throw ExceptionResultEnum.ERROR.exception("同课程同教学班，只允许设置一个任课老师");
//            }
//        }

        if (SystemConstant.longNotNull(basicExamStudent.getId())) {
            // 编辑 （学号不可更改）
            BasicExamStudent dbBasicExamStudent = this.getById(basicExamStudent.getId());
            if (Objects.isNull(dbBasicExamStudent)) {
                throw ExceptionResultEnum.ERROR.exception("未查询到此考生");
            }
            if (!basicExamStudent.getStudentCode().equals(dbBasicExamStudent.getStudentCode())) {
                throw ExceptionResultEnum.ERROR.exception("不允许修改学号");
            }
            if (StringUtils.isNotBlank(dbBasicExamStudent.getPaperNumber()) && !dbBasicExamStudent.getPaperNumber().equals(paperNumber)) {
                throw ExceptionResultEnum.ERROR.exception("不允许修改试卷编号");
            }
            if (StringUtils.isNotBlank(dbBasicExamStudent.getPaperType()) && dbBasicExamStudent.getPaperNumber().equals(paperType)) {
                throw ExceptionResultEnum.ERROR.exception("不允许修改卷型");
            }
            basicExamStudent.updateInfo(requestUserId);
        } else {
            // 新增 （学号在考试课程下唯一）
            if (this.count(new QueryWrapper<BasicExamStudent>().lambda()
                    .eq(BasicExamStudent::getExamId, basicExamStudent.getExamId()).eq(BasicExamStudent::getCourseId, basicExamStudent.getCourseId())
                    .eq(BasicExamStudent::getStudentCode, basicExamStudent.getStudentCode())) > 0) {
                throw ExceptionResultEnum.ERROR.exception(String.format("学号[%s]考生已存在", basicExamStudent.getStudentCode()));
            }
            basicExamStudent.insertInfo(requestUserId);
        }

        parserRequiredField(basicExamStudent);
        return basicExamStudent;
    }

    private void parserRequiredField(BasicExamStudent basicExamStudent) {
        // 组装requiredFields。extendFields前端传
        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(basicExamStudent.getSchoolId());
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }
        Map<String, EnumResult> enumResultMap = basicExamRule.getRequiredFieldList().stream().filter(m -> m.getEnable()).collect(Collectors.toMap(EnumResult::getCode, m -> m));
        List<CodeNameEnableDisabledValue> requiredFields = new ArrayList<>();
        Field[] declaredFields = basicExamStudent.getClass().getDeclaredFields();
        try {
            boolean examDateHasValue = false;
            for (Field declaredField : declaredFields) {
                declaredField.setAccessible(true);
                String fieldName = declaredField.getName();
                if ("courseId".equals(fieldName)) {
                    BasicCourse basicCourse = basicCourseService.getById(basicExamStudent.getCourseId());
                    if (basicCourse == null) {
                        throw ExceptionResultEnum.ERROR.exception("课程不存在");
                    }
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.COURSE_CODE.getCode(), RequiredFieldsEnum.COURSE_CODE.getName(), RequiredFieldsEnum.COURSE_CODE.getEnable(), RequiredFieldsEnum.COURSE_CODE.getDisabled(), basicCourse.getCode()));
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.COURSE_NAME.getCode(), RequiredFieldsEnum.COURSE_NAME.getName(), RequiredFieldsEnum.COURSE_NAME.getEnable(), RequiredFieldsEnum.COURSE_NAME.getDisabled(), basicCourse.getName()));
                    SysOrg sysOrg = sysOrgService.getById(basicCourse.getTeachingRoomId());
                    if (sysOrg == null) {
                        throw ExceptionResultEnum.ERROR.exception("课程开课学院不存在");
                    }
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.TEACHING_ROOM_NAME.getCode(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getName(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getEnable(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getDisabled(), sysOrg.getName()));
                } else if ("teacherId".equals(fieldName)) {
                    String loginName = null;
                    String realName = null;
                    if (basicExamStudent.getTeacherId() != null) {
                        SysUser sysUser = sysUserService.getById(basicExamStudent.getTeacherId());
                        if (sysUser == null) {
                            throw ExceptionResultEnum.ERROR.exception("任课老师不存在");
                        }
                        loginName = sysUser.getLoginName();
                        realName = sysUser.getRealName();
                    }
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.TEACHER_CODE.getCode(), RequiredFieldsEnum.TEACHER_CODE.getName(), RequiredFieldsEnum.TEACHER_CODE.getEnable(), RequiredFieldsEnum.TEACHER_CODE.getDisabled(), loginName));
                    requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.TEACHER_NAME.getCode(), RequiredFieldsEnum.TEACHER_NAME.getName(), RequiredFieldsEnum.TEACHER_NAME.getEnable(), RequiredFieldsEnum.TEACHER_NAME.getDisabled(), realName));

                } else if ("examStartTime".equals(fieldName) || "examEndTime".equals(fieldName)) {
                    Long examStartTime = basicExamStudent.getExamStartTime();
                    Long examEndTime = basicExamStudent.getExamEndTime();
                    if (SystemConstant.longNotNull(examStartTime) && SystemConstant.longNotNull(examEndTime) && !examDateHasValue) {
                        Map<String, Object> dateMap = ConvertUtil.analyzeDateAndTime(examStartTime, examEndTime);
                        String date = String.valueOf(dateMap.get("date"));
                        String time = String.valueOf(dateMap.get("time"));
                        requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.EXAM_DATE.getCode(), RequiredFieldsEnum.EXAM_DATE.getName(), RequiredFieldsEnum.EXAM_DATE.getEnable(), RequiredFieldsEnum.EXAM_DATE.getDisabled(), date));
                        requiredFields.add(new CodeNameEnableDisabledValue(RequiredFieldsEnum.EXAM_TIME.getCode(), RequiredFieldsEnum.EXAM_TIME.getName(), RequiredFieldsEnum.EXAM_TIME.getEnable(), RequiredFieldsEnum.EXAM_TIME.getDisabled(), time));
                        examDateHasValue = true;
                    }
                } else {
                    if (enumResultMap.containsKey(fieldName)) {
                        requiredFields.add(new CodeNameEnableDisabledValue(fieldName, enumResultMap.get(fieldName).getName(), enumResultMap.get(fieldName).getEnable(), enumResultMap.get(fieldName).getDisabled(), declaredField.get(basicExamStudent) != null ? declaredField.get(basicExamStudent).toString() : ""));
                    }
                }
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
        basicExamStudent.setRequiredFields(JSON.toJSONString(requiredFields));
    }

    @Override
    public List<BasicExamStudent> listByExamIdAndCourseIdAndPaperNumber(Long examId, Long courseId, String paperNumber) {
        QueryWrapper<BasicExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getCourseId, courseId)
                .eq(BasicExamStudent::getPaperNumber, paperNumber)
                .orderByAsc(BasicExamStudent::getStudentCode);
        return this.list(queryWrapper);
    }

    @Override
    public List<BasicExamStudentResult> listByParams(BasicExamStudentQuery basicExamStudentQuery) {
        return this.baseMapper.listByParams(basicExamStudentQuery);
    }

    @Override
    public boolean updateStatus(Long id, BasicExamStudentStatusEnum status) {
        return this.update(new UpdateWrapper<BasicExamStudent>().lambda().set(BasicExamStudent::getStatus, status).eq(BasicExamStudent::getId, id));
    }

    @Transactional
    @Override
    public void updateTeacherIdById(List<BasicExamStudent> basicExamStudentList) {
        for (BasicExamStudent basicExamStudent : basicExamStudentList) {
            this.update(new UpdateWrapper<BasicExamStudent>().lambda()
                    .set(BasicExamStudent::getTeacherId, basicExamStudent.getTeacherId())
                    .set(BasicExamStudent::getRequiredFields, basicExamStudent.getRequiredFields())
                    .eq(BasicExamStudent::getId, basicExamStudent.getId()));
        }
    }

    @Override
    public void updatePaperNumberAndPaperTypeById(List<ExamStudent> examStudentList) {
        for (ExamStudent examStudent : examStudentList) {
            BasicExamStudent basicExamStudent = this.getById(examStudent.getBasicStudentId());
            if (StringUtils.isNoneBlank(basicExamStudent.getPaperNumber(), basicExamStudent.getPaperType()) && !examStudent.getPaperNumber().equals(basicExamStudent.getPaperNumber())) {
                throw ExceptionResultEnum.ERROR.exception(String.format("试卷编号[%s]，学号[%s]考生在试卷编号[%s]生成了印刷任务", examStudent.getPaperNumber(), basicExamStudent.getStudentCode(), basicExamStudent.getPaperNumber()));
            }
            List<CodeNameEnableValue> requiredFieldList = basicExamStudent.getRequiredFieldList();
            for (CodeNameEnableValue codeNameEnableValue : requiredFieldList) {
                if (RequiredFieldsEnum.PAPER_NUMBER.getCode().equals(codeNameEnableValue.getCode())) {
                    codeNameEnableValue.setValue(examStudent.getPaperNumber());
                }
            }
            this.update(new UpdateWrapper<BasicExamStudent>().lambda()
                    .set(BasicExamStudent::getPaperNumber, examStudent.getPaperNumber())
                    .set(BasicExamStudent::getPaperType, examStudent.getPaperType())
                    .set(BasicExamStudent::getRequiredFields, JSON.toJSONString(requiredFieldList))
                    .eq(BasicExamStudent::getId, examStudent.getBasicStudentId()));
        }
    }

    @Override
    public void updateMarkData(Long examId, List<BasicExamStudent> basicExamStudentList, Long createId) {
        BasicExam basicExam = basicExamService.getById(examId);
        Map<String, List<BasicExamStudent>> stringMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.groupingBy(BasicExamStudent::getPaperNumber));
        for (Map.Entry<String, List<BasicExamStudent>> entry : stringMap.entrySet()) {
            BasicExamStudent basicExamStudent = entry.getValue().get(0);
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumberAndPaperType(examId, entry.getKey(), null);
            if (markPaper == null && ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
                String paperType = SystemConstant.DEFAULT_PAPER_TYPE_A;
                markPaper = new MarkPaper(examId, basicExamStudent.getCourseId(), entry.getKey(), String.valueOf(System.currentTimeMillis()), createId, paperType);
                markPaperService.save(markPaper);
            }
            if (markPaper != null) {
                Set<String> secretNumberSet = new HashSet<>();
                MarkPaper finalMarkPaper = markPaper;
                entry.getValue().forEach(m -> markStudentService.updateByBasicExamStudent(m, finalMarkPaper, secretNumberSet));
                markPaperService.updateStudentCountByExamIdAndPaperNumberAndPaperType(examId, entry.getKey(), markPaper.getPaperType());
                markPaperService.updateUploadCount(examId, entry.getKey(), markStudentService.countUploadedByExamIdAndPaperNumber(examId, entry.getKey()));
            }
        }
    }

    @Override
    public List<DeleteBasicExamStudentStatusDto> getStudentMarkStatus(List<Long> idList) {
        List<BasicExamStudent> basicExamStudentList = this.listByIds(idList);
        List<DeleteBasicExamStudentStatusDto> list = new ArrayList<>();
        for (BasicExamStudent basicExamStudent : basicExamStudentList) {
            BasicCourse basicCourse = basicCourseService.getById(basicExamStudent.getCourseId());
            SysOrg sysOrg = sysOrgService.getById(basicCourse != null ? basicCourse.getTeachingRoomId() : null);
            List<MarkStudent> markStudentList = markStudentService.listByStudentId(basicExamStudent.getId());
            Map<Long, List<MarkStudent>> map = markStudentList.stream().collect(Collectors.groupingBy(MarkStudent::getBasicStudentId));
            for (Map.Entry<Long, List<MarkStudent>> entry : map.entrySet()) {
                DeleteBasicExamStudentStatusDto deleteBasicExamStudentStatusDto = new DeleteBasicExamStudentStatusDto(basicExamStudent.getStudentName(), basicExamStudent.getStudentCode(), basicCourse != null ? basicCourse.getCode() : null, basicCourse != null ? basicCourse.getName() : null, sysOrg != null ? sysOrg.getName() : null);
                deleteBasicExamStudentStatusDto.setScanStatus(entry.getValue().stream().filter(m -> m.getUpload() != null && m.getUpload()).count() > 0);
                int taskCount = 0;
                for (MarkStudent markStudent : entry.getValue()) {
                    if (taskCount > 0) {
                        break;
                    }
                    taskCount += markTaskService.countByStudentId(markStudent.getId());
                }
                deleteBasicExamStudentStatusDto.setMarkTaskStatus(taskCount > 0);
                list.add(deleteBasicExamStudentStatusDto);
            }
        }
        return list;
    }

    /**
     * 查询班级和老师信息
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @return
     */
    @Override
    public ExamStudentDto queryExamStudent(Long examId, Long courseId, String paperNumber) {
        return this.baseMapper.queryExamStudent(examId, courseId, paperNumber);
    }

    @Override
    public List<BasicExamStudent> listByExamIdAndCourseIdAndTeachClassName(Long examId, Long courseId, String teachClassName) {
        QueryWrapper<BasicExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamStudent::getExamId, examId)
                .eq(BasicExamStudent::getCourseId, courseId)
                .eq(BasicExamStudent::getTeachClassName, teachClassName);
        return this.list(queryWrapper);
    }

    @Override
    public void clearPaperNumberAndPaperType(List<Long> ids) {
        UpdateWrapper<BasicExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicExamStudent::getPaperNumber, null)
                .set(BasicExamStudent::getPaperType, null)
                .in(BasicExamStudent::getId, ids)
                .isNull(BasicExamStudent::getPaperType);
        this.update(updateWrapper);
    }
}
