package com.qmth.distributed.print.business.util;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itextpdf.text.pdf.PdfReader;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.dto.pdf.ExamStudentInfo;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.ExamNumberStyleEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.RequiredFieldsEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnable;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.bean.vo.OriginalVo;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.bean.vo.PrintPathVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicPrintConfig;
import com.qmth.teachcloud.common.entity.BasicSemester;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.BasicAttachmentService;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicSemesterService;
import com.qmth.teachcloud.common.service.FileUploadService;
import com.qmth.teachcloud.common.util.ExamTaskUtil;
import com.qmth.teachcloud.common.util.FileUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.websocket.RemoteEndpoint;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 创建pdf util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/14
 */
@Component
public class CreatePdfUtil {
    private final static Logger log = LoggerFactory.getLogger(CreatePdfUtil.class);

    @Resource
    private BasicAttachmentService basicAttachmentService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    @Lazy
    private PrintCommonService printCommonService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private BasicTemplateService basicTemplateService;
    @Resource
    private BasicSemesterService basicSemesterService;
    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private ExamPrintPlanService examPrintPlanService;
    @Resource
    private BasicPrintConfigService basicPrintConfigService;
    @Resource
    private FileUploadService fileUploadService;

    /**
     * 创建登记表
     *
     * @param examDetail
     * @param basicAttachment
     * @param printCount
     * @param fileTempList
     * @throws Exception
     */
    public void createCheckIn(ExamDetail examDetail, BasicAttachment basicAttachment, Integer printCount, List<File> fileTempList) throws Exception {
        Optional.ofNullable(basicAttachment).orElseThrow(() -> ExceptionResultEnum.ATTACHMENT_IS_NULL.exception());

        String type = basicAttachment.getType();
        FilePathVo filePathVo = JSON.parseObject(basicAttachment.getPath(), FilePathVo.class);

        if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.PDF_PREFIX)) {
            String filePath = filePathVo.getPath();
            PrintPathVo printPathVo = new PrintPathVo(ClassifyEnum.CHECK_IN, filePathVo.getUploadType(), filePathVo.getType(), filePath, basicAttachment.getMd5(), null, null);
            examDetail.setAttachmentPath(printCommonService.parseAttachmentPath(examDetail.getAttachmentPath(), printPathVo));

//            for (int i = 0; i < printCount; i++) {
//                ordinaryPdfList.add(new PdfDto(filePath, PageSizeEnum.A4, 0));
//            }
        } else {
            File htmlFileTemp = SystemConstant.getFileTempVar(SystemConstant.HTML_PREFIX);
            fileUploadService.downloadFile(basicAttachment.getId(), htmlFileTemp.getPath());
            BasicAttachment attachment = printCommonService.saveAttachmentPdfFromHtml(String.valueOf(basicAttachment.getId()), htmlFileTemp, examDetail.getCreateId(), null, fileTempList, PageSizeEnum.A4);
            FilePathVo filePathVo1 = JSON.parseObject(attachment.getPath(), FilePathVo.class);
            PrintPathVo printPathVo = new PrintPathVo(ClassifyEnum.CHECK_IN, filePathVo1.getUploadType(), filePathVo1.getType(), filePathVo1.getPath(), attachment.getMd5(), null, null);
            examDetail.setAttachmentPath(printCommonService.parseAttachmentPath(examDetail.getAttachmentPath(), printPathVo));
        }
    }


    /**
     * 生成卷袋贴
     */
    public void createPaperPackage(Long templateId, String schoolName, ExamDetail examDetail, List<ExamStudentInfo> examStudentInfoList, Integer backupCount, Integer printCount, List<ExamDetailCourse> examDetailCourseList, Long examId) {
        BasicTemplate basicTemplate = basicTemplateService.getById(templateId);
        if (Objects.isNull(basicTemplate)) {
            throw ExceptionResultEnum.ERROR.exception("印品数据有误");
        }

        JSONObject object = JSON.parseObject(basicTemplate.getDisplayRange());

        PdfPackageDto pdfPackageDto = new PdfPackageDto();
        pdfPackageDto.setTitle(schoolName);
        pdfPackageDto.setPackageNumber(examDetail.getPackageCode());

        boolean isTag;
        for (ExamDetailCourse detailCourse : examDetailCourseList) {
            ExamTask examTask = examTaskService.getByExamIdAndCourseIdAndPaperNumber(examId, detailCourse.getCourseId(), detailCourse.getPaperNumber());
            ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamTaskId(examTask.getId());
            List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds());
            for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                int pages = paperInfoVo.getPages();
                isTag = pages > 2;
                pdfPackageDto.setTag(isTag);
                if (isTag) {
                    break;
                }
            }
        }

        // 标题信息
        List<CodeNameEnable> objectTitleList = JSON.parseArray(object.getString("title"), CodeNameEnable.class).stream().filter(m -> m.getEnable()).collect(Collectors.toList());
        List<CodeNameEnableValue> titlePlate = new ArrayList<>();
        CodeNameEnableValue codeNameEnableValue;
        for (CodeNameEnable jsonObject : objectTitleList) {
            String code = jsonObject.getCode();
            String name = jsonObject.getName();
            BasicSemester basicSemester = basicSemesterService.selectByExamId(examId);
            codeNameEnableValue = new CodeNameEnableValue(code, name, jsonObject.getEnable(), "semesterName".equals(code) && basicSemester != null ? basicSemester.getName() + "  课程考试" : null);
            titlePlate.add(codeNameEnableValue);
        }
        pdfPackageDto.setTitlePlate(titlePlate);

        List<CodeNameEnable> objectList = JSON.parseArray(object.getString("basic"), CodeNameEnable.class).stream().filter(m -> m.getEnable()).collect(Collectors.toList());

        boolean isCourseHasTwo = objectList.stream().filter(m -> RequiredFieldsEnum.COURSE_CODE.getCode().equals(m.getCode()) || RequiredFieldsEnum.COURSE_NAME.getCode().equals(m.getCode())).count() == 2;
        boolean isDateHasTwo = objectList.stream().filter(m -> RequiredFieldsEnum.EXAM_DATE.getCode().equals(m.getCode()) || RequiredFieldsEnum.EXAM_TIME.getCode().equals(m.getCode())).count() == 2;

        Set<List<CodeNameEnableValue>> examStudentFieldSet = examStudentInfoList.stream().map(m -> m.getFieldList()).collect(Collectors.toSet());

        boolean isCourseFill = false;
        boolean isDateFill = false;
        CodeNameEnableValue basicValue;
        // 基础信息
        List<CodeNameEnableValue> basicPlate = new ArrayList<>();
        for (CodeNameEnable jsonObject : objectList) {
            String code = jsonObject.getCode();
            String name = jsonObject.getName();

            if (RequiredFieldsEnum.COURSE_CODE.getCode().equals(code) || RequiredFieldsEnum.COURSE_NAME.getCode().equals(code)) {
                if (isCourseHasTwo) {
                    if (isCourseFill) {
                        continue;
                    }
                    List<String> courseNames = examDetailCourseList.stream().map(m -> String.format("%s（%s）", m.getCourseName(), m.getCourseCode())).collect(Collectors.toList());
                    basicValue = new CodeNameEnableValue(RequiredFieldsEnum.COURSE_NAME.getCode(), RequiredFieldsEnum.COURSE_NAME.getName(), true, String.join(SystemConstant.COMMA_OF_CHINESE, courseNames));
                    isCourseFill = true;
                } else {
                    basicValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
                }
            } else if (RequiredFieldsEnum.EXAM_DATE.getCode().equals(code) || RequiredFieldsEnum.EXAM_TIME.getCode().equals(code)) {
                if (isDateHasTwo) {
                    if (isDateFill) {
                        continue;
                    }
                    String startDate = DateUtil.format(new Date(examDetail.getExamStartTime()), SystemConstant.DATE_PATTERN);
                    String startTime = DateUtil.format(new Date(examDetail.getExamStartTime()), SystemConstant.TIME_PATTERN);
                    String endTime = DateUtil.format(new Date(examDetail.getExamEndTime()), SystemConstant.TIME_PATTERN);
                    basicValue = new CodeNameEnableValue(RequiredFieldsEnum.EXAM_TIME.getCode(), RequiredFieldsEnum.EXAM_TIME.getName(), true, startDate + " " + startTime + SystemConstant.HYPHEN + endTime);
                    isDateFill = true;
                } else {
                    basicValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
                }
            } else if ("paperCount".equals(code)) {
                basicValue = new CodeNameEnableValue(code, name, true, examStudentInfoList.size() + " + " + backupCount);
            } else if ("examCount".equals(code)) {
                basicValue = new CodeNameEnableValue(code, name, true, String.valueOf(examStudentInfoList.size()));
            } else if ("actualExamCount".equals(code)) {
                basicValue = new CodeNameEnableValue(code, name, true, "");
            } else {
                basicValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
            }
            basicPlate.add(basicValue);
        }
        pdfPackageDto.setBasicPlate(basicPlate);
        printCommonService.saveAttachmentPackagePdf(pdfPackageDto, examDetail, printCount);
    }

    /**
     * 创建签到表
     */
    public void createSignBook(Long templateId, String schoolName, ExamDetail examDetail, List<ExamStudentInfo> examStudentInfoList, Integer printCount, List<ExamDetailCourse> examDetailCourseList) {
        BasicTemplate basicTemplate = basicTemplateService.getById(templateId);
        if (Objects.isNull(basicTemplate)) {
            throw ExceptionResultEnum.ERROR.exception("印品数据有误");
        }
        PdfSignDto pdfFillDto = new PdfSignDto();
        pdfFillDto.setTitle(schoolName + "签到表");
        pdfFillDto.setPackageNumber(examDetail.getPackageCode());
        pdfFillDto.setTextDesc(basicTemplate.getTextDesc());

        JSONObject jsonObject = JSON.parseObject(basicTemplate.getDisplayRange());
        List<CodeNameEnable> basicList = JSON.parseArray(jsonObject.getString("basic"), CodeNameEnable.class).stream().filter(m -> m.getEnable()).collect(Collectors.toList());

        boolean isCourseHasTwo = basicList.stream().filter(m -> RequiredFieldsEnum.COURSE_CODE.getCode().equals(m.getCode()) || RequiredFieldsEnum.COURSE_NAME.getCode().equals(m.getCode())).count() == 2;
        boolean isDateHasTwo = basicList.stream().filter(m -> RequiredFieldsEnum.EXAM_DATE.getCode().equals(m.getCode()) || RequiredFieldsEnum.EXAM_TIME.getCode().equals(m.getCode())).count() == 2;
        boolean isPlaceHasTwo = basicList.stream().filter(m -> RequiredFieldsEnum.EXAM_PLACE.getCode().equals(m.getCode()) || RequiredFieldsEnum.EXAM_ROOM.getCode().equals(m.getCode())).count() == 2;

        Set<List<CodeNameEnableValue>> examStudentFieldSet = examStudentInfoList.stream().map(m -> m.getFieldList()).collect(Collectors.toSet());
        // 基础信息
        List<CodeNameEnableValue> basicPlate = new ArrayList<>();
        boolean isCourseFill = false;
        boolean isDateFill = false;
        boolean isPlaceFill = false;
        CodeNameEnableValue codeNameEnableValue;
        for (CodeNameEnable basicObject : basicList) {
            String code = basicObject.getCode();
            String name = basicObject.getName();

            if (RequiredFieldsEnum.COURSE_CODE.getCode().equals(code) || RequiredFieldsEnum.COURSE_NAME.getCode().equals(code)) {
                if (isCourseHasTwo) {
                    if (isCourseFill) {
                        continue;
                    }
                    List<String> courseNames = examDetailCourseList.stream().map(m -> String.format("%s（%s）", m.getCourseName(), m.getCourseCode())).collect(Collectors.toList());
                    codeNameEnableValue = new CodeNameEnableValue(RequiredFieldsEnum.COURSE_NAME.getCode(), RequiredFieldsEnum.COURSE_NAME.getName(), true, String.join(SystemConstant.COMMA_OF_CHINESE, courseNames));
                    isCourseFill = true;
                } else {
                    codeNameEnableValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
                }

            } else if (RequiredFieldsEnum.EXAM_DATE.getCode().equals(code) || RequiredFieldsEnum.EXAM_TIME.getCode().equals(code)) {
                if (isDateHasTwo) {
                    if (isDateFill) {
                        continue;
                    }
                    String startDate = DateUtil.format(new Date(examDetail.getExamStartTime()), SystemConstant.DATE_PATTERN);
                    String startTime = DateUtil.format(new Date(examDetail.getExamStartTime()), SystemConstant.TIME_PATTERN);
                    String endTime = DateUtil.format(new Date(examDetail.getExamEndTime()), SystemConstant.TIME_PATTERN);
                    codeNameEnableValue = new CodeNameEnableValue(RequiredFieldsEnum.EXAM_TIME.getCode(), RequiredFieldsEnum.EXAM_TIME.getName(), true, startDate + " " + startTime + SystemConstant.HYPHEN + endTime);
                    isDateFill = true;
                } else {
                    codeNameEnableValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
                }
            } else if (RequiredFieldsEnum.EXAM_PLACE.getCode().equals(code) || RequiredFieldsEnum.EXAM_ROOM.getCode().equals(code)) {
                if (isPlaceHasTwo) {
                    if (isPlaceFill) {
                        continue;
                    }
                    StringJoiner stringJoiner = new StringJoiner("");
                    if (StringUtils.isNotBlank(examDetail.getExamPlace())) {
                        stringJoiner.add(examDetail.getExamPlace());
                    }
                    if (StringUtils.isNotBlank(examDetail.getExamRoom())) {
                        stringJoiner.add(examDetail.getExamRoom());
                    }
                    codeNameEnableValue = new CodeNameEnableValue(RequiredFieldsEnum.EXAM_ROOM.getCode(), RequiredFieldsEnum.EXAM_ROOM.getName(), true, stringJoiner.toString());
                    isPlaceFill = true;
                } else {
                    codeNameEnableValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
                }
            } else if ("examCount".equals(code)) {
                codeNameEnableValue = new CodeNameEnableValue(code, name, true, String.valueOf(examStudentInfoList.size()));
            } else if ("actualExamCount".equals(code)) {
                codeNameEnableValue = new CodeNameEnableValue(code, name, true, "");
            } else {
                codeNameEnableValue = defaultCodeNameEnableValue(code, name, examStudentFieldSet);
            }
            basicPlate.add(codeNameEnableValue);
        }
        pdfFillDto.setBasicPlate(basicPlate);

        // 表头信息
        List<CodeNameEnable> tableList = JSON.parseArray(jsonObject.getString("table"), CodeNameEnable.class).stream().filter(m -> m.getEnable()).collect(Collectors.toList());
        Map<String, String> studentHeadPlateMap = new LinkedHashMap<>();
        for (CodeNameEnable object : tableList) {
            studentHeadPlateMap.put(object.getCode(), object.getName());
        }
        studentHeadPlateMap.put("studentSign", "签名");
        pdfFillDto.setStudentHeadPlate(studentHeadPlateMap);

        // 考生信息
        List<Map<String, String>> studentPlate = new ArrayList<>();
        for (ExamStudentInfo examStudentInfo : examStudentInfoList) {
            List<CodeNameEnableValue> fieldList = examStudentInfo.getFieldList();
            Map<String, String> studentMap = new LinkedHashMap<>();
            for (Map.Entry<String, String> entry : studentHeadPlateMap.entrySet()) {
                Optional<CodeNameEnableValue> optional = fieldList.stream().filter(m -> m.getCode().equals(entry.getKey())).findFirst();
                studentMap.put(entry.getKey(), optional.isPresent() ? optional.get().getValue() : "");
            }
            studentPlate.add(studentMap);
        }
        pdfFillDto.setStudentPlate(studentPlate);

        printCommonService.saveAttachmentSignPdf(pdfFillDto, examDetail, printCount, basicTemplate);
    }

    private CodeNameEnableValue defaultCodeNameEnableValue(String code, String name, Set<List<CodeNameEnableValue>> examStudentFieldSet) {
        Set<String> valueSet = new HashSet<>();
        for (List<CodeNameEnableValue> codeNameEnableValues : examStudentFieldSet) {
            CollectionUtils.addAll(valueSet, codeNameEnableValues.stream().filter(m -> code.equals(m.getCode())).map(m -> m.getValue()).collect(Collectors.toSet()));
        }
        return new CodeNameEnableValue(code, name, true, CollectionUtils.isNotEmpty(valueSet) ? String.join(SystemConstant.COMMA_OF_CHINESE, valueSet) : "");
    }

    /**
     * 题卡html内容，所有参数置空
     *
     * @param htmlContent
     * @return
     */
    public String replaceBlankHtmlContent(String htmlContent, Long courseId) {
        BasicCourse basicCourse = courseId != null ? basicCourseService.getById(courseId) : null;
        // 替换条码参数
        htmlContent = htmlContent.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
        htmlContent = htmlContent.replaceAll("\\$\\{courseCodeBarcodeSrc\\}", basicCourse != null ? "data:image/png;base64," + GoogleBarCodeUtil.createBarCode(basicCourse.getCode(), false) : "");
        htmlContent = htmlContent.replaceAll("\\$\\{courseCodeBarcodeName\\}", basicCourse != null ? basicCourse.getCode() : "");
        // 正则匹配所有参数，填入空值
        htmlContent = htmlContent.replaceAll("\\$\\{[A-Za-z0-9]+\\}", "");
        return htmlContent;

    }

    /**
     * 获取试卷pdf
     *
     * @param stuPaperType   考生卷型
     * @param paperPdfDto    抽取的试卷文件信息
     * @param createPdfDto   生成pdf
     * @param waterMarkNames 水印内容
     */
    public PdfDto getExamStudentPaperPdf(String stuPaperType, List<PaperPdfDto> paperPdfDto, CreatePdfDto createPdfDto, String[] waterMarkNames) throws IOException {
        Set<Integer> pagesList = new HashSet<>();
        if (!CollectionUtils.isEmpty(paperPdfDto)) {
            for (PaperPdfDto dto : paperPdfDto) {
                if (stuPaperType.equals(dto.getType())) {
                    int pages = dto.getPages();
                    pagesList.add(pages);
                    PdfDto pdfDto = PdfUtil.addPdfPage(dto.getFile());
                    File file = paperPdfDto.size() > 1 ? PdfUtil.addWaterMark(dto.getFile(), waterMarkNames, 0.5f, 12, 0) : dto.getFile();
                    createPdfDto.getPaperPdfList().add(new PdfDto(file.getPath(), dto.getPageSize(), pdfDto.getPageCount()));
                }
            }
            int pageCount = pagesList.stream().mapToInt(Integer::intValue).sum();
            return new PdfDto(paperPdfDto.get(0).getPageSize(), pageCount);
        } else {
            return null;
        }
    }

    /**
     * 获取备份试卷pdf
     *
     * @param paperPdfDto  抽取的试卷文件信息
     * @param backupCount  备份数量
     * @param createPdfDto 生成pdf对象
     */
    public PdfDto getBackupPaperPdfList(List<PaperPdfDto> paperPdfDto, Integer backupCount, CreatePdfDto createPdfDto) throws
            IOException {
        Set<Integer> pagesList = new HashSet<>();
        boolean tag = false;
        if (CollectionUtils.isNotEmpty(paperPdfDto)) {
            for (PaperPdfDto dto : paperPdfDto) {
                int pages = dto.getPages();
                pagesList.add(pages);
                tag = pages > 2;
                PdfDto pdfDto = PdfUtil.addPdfPage(dto.getFile());
                for (int j = 1; j <= backupCount; j++) {
                    createPdfDto.getPaperPdfList().add(new PdfDto(dto.getFile().getPath(), dto.getPageSize(), pdfDto.getPageCount()));
                }
            }
            int pageCount = pagesList.stream().mapToInt(Integer::intValue).sum();
            return new PdfDto(paperPdfDto.get(0).getPageSize(), pageCount, tag);
        } else {
            return null;
        }
    }

    /**
     * 获取考试试卷
     *
     * @param paperType       抽取的试卷型（多个用,分隔）
     * @param paperInfoVoList 命题任务上传试卷信息
     */
    public List<PaperPdfDto> getPaperPdfFile(String paperType, List<PaperInfoVo> paperInfoVoList, CreatePdfDto createPdfDto) throws Exception {
        String[] paperTypes = paperType.split(",");
        List<PaperPdfDto> paperPdfDtoList = new ArrayList<>();
        PaperPdfDto paperPdfDto;
        for (PaperInfoVo paperInfoVo : paperInfoVoList) {
            BasicAttachment basicAttachment = basicAttachmentService.getById(paperInfoVo.getAttachmentId());
            Optional.ofNullable(basicAttachment).orElseThrow(() -> ExceptionResultEnum.ATTACHMENT_IS_NULL.exception("命题任务中上传试卷文件路径不存在"));

            // 卷型
            String name = paperInfoVo.getName();
            OriginalVo original = paperInfoVo.getOriginal();
            // 返回抽中的试卷
            for (String type : paperTypes) {
                if (StringUtils.equalsIgnoreCase(name, type)) {
                    File file = SystemConstant.getFileTempVar(SystemConstant.PDF_PREFIX);
                    createPdfDto.getFileTempList().add(file);
                    fileUploadService.downloadFile(basicAttachment, file.getPath());
//                    PageSizeEnum pageSizeEnum = PageSizeEnum.valueOf(original.getPageSize());
//                    Optional.ofNullable(pageSizeEnum).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("命题任务中上传试卷文件未设置pdf格式"));
                    paperPdfDto = new PaperPdfDto(type, file, new PdfReader(new FileInputStream(file)).getNumberOfPages(), null);
                    paperPdfDtoList.add(paperPdfDto);
                    break;
                }
            }
        }
        return paperPdfDtoList;
    }

    private String randomPaperType(String[] unexposedPaperTypes, int drawCount) {
        List<String> finalTypes = new ArrayList<>();
        List<String> unexposedPaperTypesList = new ArrayList<>(Arrays.asList(unexposedPaperTypes));
        for (int i = 0; i < drawCount; i++) {
            int index = new Random().nextInt(unexposedPaperTypesList.size());
            finalTypes.add(unexposedPaperTypesList.get(index));
            unexposedPaperTypesList.remove(index);
        }

        finalTypes = finalTypes.stream().sorted(Comparator.comparing(String::valueOf)).collect(Collectors.toList());
        return String.join(",", finalTypes);
    }

    /**
     * 合并A3 pdf
     *
     * @param list
     * @return
     * @throws IOException
     */
    public String mergeA3Pdf(List<PdfDto>... list) {
        List<PdfDto> mergePdfA3List = new ArrayList<>();
        for (int i = 0; i < list.length; i++) {
            mergePdfA3List.addAll(list[i]);
        }
        List<String> pathA3List = mergePdfA3List.stream().map(PdfDto::getPath).collect(Collectors.toList());
        return PdfUtil.mergePdf(pathA3List.toArray(new String[mergePdfA3List.size()]), null);
    }

    /**
     * 合并A4 pdf
     *
     * @param list
     * @return
     * @throws IOException
     */
    public String mergeA4Pdf(List<File> fileTempList, List<PdfDto>... list) {
        List<PdfDto> mergePdfA4List = new ArrayList<>();
        for (int i = 0; i < list.length; i++) {
            mergePdfA4List.addAll(list[i]);
        }
        List<String> pathA4List = mergePdfA4List.stream().map(PdfDto::getPath).collect(Collectors.toList());
        return PdfUtil.mergePdf(pathA4List.toArray(new String[mergePdfA4List.size()]), null, fileTempList);
    }

    /**
     * 合并pdf后保存数据库
     *
     * @param dirNamePaper
     * @param dirNameCardA3
     * @param tbTaskPdf
     * @return
     * @throws Exception
     */
    @Transactional
    public void mergePdfSaveDb(String dirNamePaper,
                               String dirNameCardA3,
                               TBTaskPdf tbTaskPdf) throws Exception {
        ExamDetail examDetail = examDetailService.getById(tbTaskPdf.getId());
        BasicPrintConfig basicPrintConfig = basicPrintConfigService.getByExamId(tbTaskPdf.getExamId());
        // 试卷+题卡
        if (StringUtils.isNotBlank(dirNamePaper)) {
            File localPaperPdfFile = new File(FileUtil.replaceSplit(dirNamePaper));
            examDetail.setPaperPages(PdfUtil.getPdfPages(localPaperPdfFile));

            String uploadFileName = PdfUtil.md5FileName(CreatePdfTypeEnum.PAPER.name() + tbTaskPdf.getId()) + SystemConstant.PDF_PREFIX;
            FilePathVo filePathVo = fileUploadService.uploadFile(localPaperPdfFile, UploadFileEnum.PDF, uploadFileName);
            BasicAttachment basicAttachment = new BasicAttachment(JSON.toJSONString(filePathVo), localPaperPdfFile.getName(), SystemConstant.PDF_PREFIX, new BigDecimal(localPaperPdfFile.length()), filePathVo.getMd5(), tbTaskPdf.getCreateId());
            basicAttachmentService.save(basicAttachment);
            examDetail.setAttachmentId(basicAttachment.getId());
        }
        // 题卡
        if (StringUtils.isNotBlank(dirNameCardA3)) {
            File localA3PdfCardFile = new File(FileUtil.replaceSplit(dirNameCardA3));
            examDetail.setCardPages(PdfUtil.getPdfPages(localA3PdfCardFile));

            String uploadFileName = PdfUtil.md5FileName(CreatePdfTypeEnum.CARD_A3.name() + tbTaskPdf.getId()) + SystemConstant.PDF_PREFIX;
            FilePathVo filePathVo = fileUploadService.uploadFile(localA3PdfCardFile, UploadFileEnum.PDF, uploadFileName);
            BasicAttachment basicAttachment = new BasicAttachment(JSON.toJSONString(filePathVo), localA3PdfCardFile.getName(), SystemConstant.PDF_PREFIX, new BigDecimal(localA3PdfCardFile.length()), filePathVo.getMd5(), tbTaskPdf.getCreateId());
            basicAttachmentService.save(basicAttachment);
            examDetail.setCardAttachmentId(basicAttachment.getId());
        }
        if (PrintMethodEnum.AUTO == basicPrintConfig.getPrintMethod()) {
            examDetail.setStatus(ExamDetailStatusEnum.WAITING);
        } else {
            examDetail.setStatus(ExamDetailStatusEnum.READY);
        }
        examDetail.setNormal(true);
        examDetailService.saveOrUpdate(examDetail);

        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTaskPdf.getPrintPlanId());
        if (PrintMethodEnum.AUTO == basicPrintConfig.getPrintMethod()) {
            examPrintPlan.setStatus(PrintPlanStatusEnum.PRINTING);
            examPrintPlanService.updateById(examPrintPlan);
        }
    }

    /**
     * 考生题卡html
     *
     * @param studentHtml        题卡html内容
     * @param examStudent        考生对象
     * @param examDetail         考场对象
     * @param userId             当前用户ID
     * @param examStudentPdfList 考生题卡集合
     * @param fileTempList
     * @param basicCardRule      题卡规则
     */
    public BasicAttachment examStudentHtml(String studentHtml, ExamStudentInfo examStudent, String paperType, ExamDetail examDetail,
                                           Long userId, List<PdfDto> examStudentPdfList, List<File> fileTempList, BasicCardRule basicCardRule) throws IOException {
        if (basicCardRule != null) {
            if (ExamNumberStyleEnum.PRINT.equals(basicCardRule.getExamNumberStyle())) {
                // 生成学生考号条码并将图片转成base64
                studentHtml = studentHtml.replaceAll("\\$\\{studentCodeBarcode\\}", examStudent != null && examStudent.getStudentCode() != null ? GoogleBarCodeUtil.createBarCode(examStudent.getStudentCode(), false) : "");
                studentHtml = studentHtml.replaceAll("\\$\\{ticketNumberBarcode\\}", "");
                studentHtml = studentHtml.replaceAll("\\$\\{examNumber\\}", examStudent != null && examStudent.getStudentCode() != null ? GoogleBarCodeUtil.createBarCode(examStudent.getStudentCode(), false) : "");
                studentHtml = studentHtml.replaceAll("\\$\\{examNumberStr\\}", examStudent != null && examStudent.getStudentCode() != null ? examStudent.getStudentCode() : "");
                studentHtml = studentHtml.replaceAll("\\$\\{courseCodeBarcodeSrc\\}", examStudent != null && examStudent.getCourseCode() != null ? "data:image/png;base64," + GoogleBarCodeUtil.createBarCode(examStudent.getCourseCode(), false) : "");
                studentHtml = studentHtml.replaceAll("\\$\\{courseCodeBarcodeName\\}", examStudent != null && examStudent.getCourseCode() != null ? examStudent.getCourseCode() : "");
                // 随机生成学生试卷条码并将图片转成base64
                studentHtml = studentHtml.replaceAll("\\$\\{paperType\\}", examStudent != null && examStudent.getPaperType() != null ? GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(examStudent.getPaperType()), false) : StringUtils.isNotBlank(paperType) ? GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false) : "");
                // 替换考生卷型
                studentHtml = studentHtml.replaceAll("\\$\\{paperTypeName\\}", examStudent != null && examStudent.getPaperType() != null ? examStudent.getPaperType() : StringUtils.isNotBlank(paperType) ? paperType : "");

                if (examStudent != null) {
                    for (CodeNameEnableValue extendDto : examStudent.getFieldList()) {
                        String value = extendDto.getValue();
                        // 考点和考场数据，用考场表中的值
                        if (RequiredFieldsEnum.EXAM_PLACE.getCode().equals(extendDto.getCode())) {
                            value = examDetail.getExamPlace();
                        } else if (RequiredFieldsEnum.EXAM_ROOM.getCode().equals(extendDto.getCode())) {
                            value = examDetail.getExamRoom();
                        }
                        studentHtml = studentHtml.replaceAll("\\$\\{" + extendDto.getCode() + "\\}", value);
                    }
                }
            } else if (ExamNumberStyleEnum.FILL.equals(basicCardRule.getExamNumberStyle())) {
                studentHtml = studentHtml.replaceAll("\\$\\{courseCodeBarcodeSrc\\}", examStudent != null && examStudent.getCourseCode() != null ? "data:image/png;base64," + GoogleBarCodeUtil.createBarCode(examStudent.getCourseCode(), false) : "");
                studentHtml = studentHtml.replaceAll("\\$\\{courseCodeBarcodeName\\}", examStudent != null && examStudent.getCourseCode() != null ? examStudent.getCourseCode() : "");
                String fillFields = basicCardRule.getFillFields();
                if (StringUtils.isNotBlank(fillFields)) {
                    if (examStudent != null) {
                        for (String s : fillFields.split(",")) {
                            Optional<CodeNameEnableValue> extendDto = examStudent.getFieldList().stream().filter(m -> m.getCode().equals(s)).findFirst();
                            if (extendDto.isPresent()) {
                                CodeNameEnableValue studentExtendDto = extendDto.get();
                                String value = String.valueOf(studentExtendDto.getValue());
                                // 考点和考场数据，用考场表中的值
                                if (RequiredFieldsEnum.EXAM_PLACE.getCode().equals(studentExtendDto.getCode())) {
                                    value = examDetail.getExamPlace();
                                } else if (RequiredFieldsEnum.EXAM_ROOM.getCode().equals(studentExtendDto.getCode())) {
                                    value = examDetail.getExamRoom();
                                }
                                studentHtml = studentHtml.replaceAll("\\$\\{" + studentExtendDto.getCode() + "\\}", value);
                            }
                        }
                    }
                }
            }
        } else {
            // 替换条码参数
            studentHtml = studentHtml.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
            studentHtml = studentHtml.replaceAll("\\$\\{examNumberStr\\}", "");
            studentHtml = studentHtml.replaceAll("<img src=\"data:image/png;base64,\\$\\{courseCodeBarcodeSrc\\}\">", "");
            studentHtml = studentHtml.replaceAll("\\$\\{courseCodeBarcodeName\\}", "");
        }

        studentHtml = studentHtml.replaceAll("\\$\\{packageCodeDom\\}", "");
        // 将其它未匹配到值的参数，填入空值
        studentHtml = studentHtml.replaceAll("\\$\\{[A-Za-z0-9]+\\}", "");
        //学生题卡
        BasicAttachment examStudentAttachment = printCommonService.saveAttachmentPdfFromHtml(examStudent != null ? examStudent.getExamStudentId() + "|" + examStudent.getStudentCode() : SystemConstant.getNanoId(), studentHtml, userId, examStudentPdfList, fileTempList);
        if (examStudent != null) {
            examStudent.setAttachmentId(examStudentAttachment.getId());
        }
        return examStudentAttachment;
    }

    /**
     * 通用题卡html
     *
     * @param examCard          examCard
     * @param paperType         卷型
     * @param basicCardRule     题卡规则
     * @param backupPackageCode 备用题卡条码号
     */
    public String replaceBackupCardHtmlParam(String courseCode, ExamCard examCard, String paperType, BasicCardRule basicCardRule, String backupPackageCode) {
        String htmlContent = examCard.getHtmlContent();
        // 替换条码参数
        htmlContent = htmlContent.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
        htmlContent = htmlContent.replaceAll("\\$\\{examNumberStr\\}", "");

        htmlContent = htmlContent.replaceAll("\\$\\{courseCodeBarcodeSrc\\}", StringUtils.isNotBlank(courseCode) ? "data:image/png;base64," + GoogleBarCodeUtil.createBarCode(courseCode, false) : "");
        htmlContent = htmlContent.replaceAll("\\$\\{courseCodeBarcodeName\\}", StringUtils.isNotBlank(courseCode) ? courseCode : "");

        //替换卷型参数
        htmlContent = htmlContent.replaceAll("\\$\\{paperTypeName\\}", StringUtils.isBlank(paperType) ? "" : paperType);
        //替换卷型条码（base64）
        if (Objects.nonNull(paperType)) {
            htmlContent = htmlContent.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
        }
        // 替换考生变量参数
        if (Objects.nonNull(basicCardRule) && Objects.nonNull(basicCardRule.getExtendFields())) {
            JSONArray jsonObjectRequired = (JSONArray) JSONArray.parse(basicCardRule.getRequiredFields());//必选字段
            if (Objects.nonNull(jsonObjectRequired)) {
                int i = 0;
                while (i < jsonObjectRequired.size()) {
                    JSONObject object = (JSONObject) jsonObjectRequired.get(i);
                    htmlContent = htmlContent.replaceAll("\\$\\{" + object.get("code") + "\\}", "");
                    i++;
                }
            }

            JSONArray jsonObjectExtend = (JSONArray) JSONArray.parse(basicCardRule.getExtendFields());//扩展字段
            if (Objects.nonNull(jsonObjectExtend)) {
                int i = 0;
                while (i < jsonObjectExtend.size()) {
                    JSONObject object = (JSONObject) jsonObjectExtend.get(i);
                    htmlContent = htmlContent.replaceAll("\\$\\{" + object.get("code") + "\\}", "");
                    i++;
                }
            }
        }

        if (StringUtils.isNotBlank(backupPackageCode)) {
            //通用题卡生成卷袋贴条码
            String packageCodeImg = GoogleBarCodeUtil.createBarCode(backupPackageCode, false);
            // 上传通卡
            if (CardCreateMethodEnum.UPLOAD.equals(examCard.getCreateMethod())) {
                if (htmlContent.contains("<div class=\"page-box page-box-0\">")) {
                    String packageCodeDiv = "<div class=\"page-box page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + backupPackageCode + "</p></div>";
                    htmlContent = htmlContent.replaceAll("<div class=\"page-box page-box-0\">", packageCodeDiv);
                } else if (htmlContent.contains("<div class=\"page-box page-box-A3 page-box-0\">")) {
                    String packageCodeDiv = "<div class=\"page-box page-box-A3 page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + backupPackageCode + "</p></div>";
                    htmlContent = htmlContent.replaceAll("<div class=\"page-box page-box-A3 page-box-0\">", packageCodeDiv);
                } else if (htmlContent.contains("<div class=\"page-box page-box-A3 page-box-0 page-box-less\">")) {
                    String packageCodeDiv = "<div class=\"page-box page-box-A3 page-box-0 page-box-less\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + backupPackageCode + "</p></div>";
                    htmlContent = htmlContent.replaceAll("<div class=\"page-box page-box-A3 page-box-0 page-box-less\">", packageCodeDiv);
                } else if (htmlContent.contains("<div class=\"page-box page-box-A4 page-box-0\">")) {
                    String packageCodeDiv = "<div class=\"page-box page-box-A4 page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + backupPackageCode + "</p></div>";
                    htmlContent = htmlContent.replaceAll("<div class=\"page-box page-box-A4 page-box-0\">", packageCodeDiv);
                } else if (htmlContent.contains("<div class=\"page-box page-box-A4 page-box-0 page-box-less\">")) {
                    String packageCodeDiv = "<div class=\"page-box page-box-A4 page-box-0 page-box-less\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + backupPackageCode + "</p></div>";
                    htmlContent = htmlContent.replaceAll("<div class=\"page-box page-box-A4 page-box-0 page-box-less\">", packageCodeDiv);
                } else if (htmlContent.contains("<div id=\"preview-page-box-0\" class=\"page-box page-box-A3 page-box-0 page-box-less\">")) {
                    String packageCodeDiv = "<div id=\"preview-page-box-0\" class=\"page-box page-box-A3 page-box-0 page-box-less\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 10px;right: 25%;margin-left: -100px;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + packageCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + backupPackageCode + "</p></div>";
                    htmlContent = htmlContent.replaceAll("<div id=\"preview-page-box-0\" class=\"page-box page-box-A3 page-box-0 page-box-less\">", packageCodeDiv);
                }
            } else {
                htmlContent = htmlContent.replaceAll("\\$\\{packageCodeDom\\}", "<img src='" + "data:image/png;base64," + packageCodeImg + "'><p>" + backupPackageCode + "</p>");
            }
        } else {
            htmlContent = htmlContent.replaceAll("\\$\\{packageCodeDom\\}", "");
        }
        return htmlContent;
    }

    public String getPaperType(Long printPlanId, Long examId, String paperNumber) {
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);
        //抽取卷型
        DrawRuleEnum drawRule = Objects.nonNull(examPrintPlan.getDrawRule()) ? examPrintPlan.getDrawRule() : DrawRuleEnum.ONE;
        ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamIdAndCourseIdAndPaperNumber(examId, null, paperNumber);
        // 单次抽取套数
        int drawCount = examTaskDetail.getDrawCount().intValue();

        String paperType = null;
        try {
            //未曝光卷型
            String unexposedPaperType = examTaskDetail.getUnexposedPaperType();
            //已曝光卷型
            String exposedPaperType = examTaskDetail.getExposedPaperType();
            String totalPaperType = examTaskDetail.getPaperType();
            String[] paperTypes;
            if (drawRule == DrawRuleEnum.ONE) {
                if (StringUtils.isBlank(unexposedPaperType)) {
                    throw ExceptionResultEnum.PAPER_ERROR.exception();
                }
                paperTypes = unexposedPaperType.split(",");
                if (paperTypes.length - drawCount < 0) {
                    throw ExceptionResultEnum.ERROR.exception("可用卷型数量小于抽取数量");
                }
            } else {
                if (StringUtils.isAllBlank(totalPaperType, unexposedPaperType)) {
                    throw ExceptionResultEnum.PAPER_ERROR.exception();
                }
                if (StringUtils.isNotBlank(unexposedPaperType)) {
                    paperTypes = unexposedPaperType.split(",");
                    // 未曝光卷型小于抽取数量，则使用所有卷型
                    if (paperTypes.length - drawCount < 0) {
                        paperTypes = totalPaperType.split(",");
                    }
                } else {
                    paperTypes = totalPaperType.split(",");
                }
            }
            paperType = randomPaperType(paperTypes, drawCount);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        }
        return paperType;
    }

}
